"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Transformer_1 = __importDefault(require("./Transformer"));
const KeyGenerator_1 = __importDefault(require("./KeyGenerator"));
const BitBufferArray_1 = __importDefault(require("./BitBufferArray"));
require("../extensions/string.extension");
class Cryptography {
    /**
     * Generates a key using the provided secret array.
     *
     * @param {number[]} secret - The secret bit array used to generate the key. Must have a length of 10.
     * @returns {Keys} - An object containing the generated keys.
     * @throws {Error} - Throws an error if the secret array does not have a length of 10.
     */
    static GenerateKey(secret) {
        const [keys, _] = KeyGenerator_1.default.Generate(secret);
        return keys;
    }
    /**
     * Encrypts the provided string using the provided keys.
     *
     * @param {string} string - The string to encrypt.
     * @param {Keys} keys - An object containing the encryption keys.
     * @returns {BitBuffer[]} - An array of BitBuffer objects representing the encrypted string.
     */
    static Encryption(string, keys) {
        const buffers = string.toBinary();
        const cipherBuffer = new BitBufferArray_1.default();
        for (const buffer of buffers) {
            const [cipherByte, _] = Transformer_1.default.Encrypt(buffer, keys);
            cipherBuffer.push(cipherByte);
        }
        return cipherBuffer;
    }
    /**
     * Decrypts the provided cipher BitBuffer array using the provided keys.
     *
     * @param {BitBuffer[]} cipherBitArray - An array of BitBuffer objects representing the cipher to decrypt.
     * @param {Keys} keys - An object containing the decryption keys.
     * @returns {string} - The decrypted string.
     */
    static Decryption(cipherBitArray, keys) {
        const plainBytes = new BitBufferArray_1.default();
        for (const bitBuffer of cipherBitArray) {
            const [plainByte, _] = Transformer_1.default.Decrypt(bitBuffer, keys);
            plainBytes.push(plainByte);
        }
        return plainBytes.toAscii();
    }
}
exports.default = Cryptography;
