"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BitBuffer_1 = __importDefault(require("./BitBuffer"));
const sbox_1 = require("../constants/sbox");
class Mapper {
    /** Utility class that provides methods for mapping bit buffers with given permutation or S-Box. */
    /**
     * Maps the given BitBuffer with the provided permutation array.
     *
     * @param {ReadonlyArray<number>} permutation - The permutation array.
     * @param {BitBuffer} buffer - The BitBuffer to be mapped.
     * @returns {BitBuffer} A new BitBuffer mapped using the provided permutation array.
     */
    static MapPermutation(permutation, buffer) {
        const result = new Array(permutation.length);
        for (let i = 0; i < permutation.length; i++) {
            result[i] = buffer.buffer[permutation[i] - 1];
        }
        return new BitBuffer_1.default(result);
    }
    /**
     * Maps the given BitBuffer with the provided S-Box array.
     *
     * @param {BitBuffer} buffer - The BitBuffer to be mapped.
     * @returns {BitBuffer} A new BitBuffer mapped using the provided S-Box array.
     */
    static MapWithSboxs(buffer) {
        const mapping = [0, 3, 1, 2];
        const left = Mapper.MapWithSbox(buffer.left, sbox_1.SBox.S0, mapping);
        const right = Mapper.MapWithSbox(buffer.right, sbox_1.SBox.S1, mapping);
        const concatenated = left.concat(right);
        return new BitBuffer_1.default(concatenated);
    }
    /**
     * Maps the given BitBuffer with the provided S-Box and mapping array.
     *
     * @private
     * @param {BitBuffer} buffer - The BitBuffer to be mapped.
     * @param {number[][]} sBox - The S-Box array.
     * @param {number[]} mapping - The mapping array.
     * @returns {number[]} An array of mapped values.
     */
    static MapWithSbox(buffer, sBox, mapping) {
        const row = parseInt(`${buffer.buffer[mapping[0]]}${buffer.buffer[mapping[1]]}`, 2);
        const column = parseInt(`${buffer.buffer[mapping[2]]}${buffer.buffer[mapping[3]]}`, 2);
        let s = sBox[row][column].toString(2);
        if (s.length === 1)
            s = `0${s}`;
        const result = new Array(s.length);
        for (let i = 0; i < s.length; i++) {
            result[i] = parseInt(s[i], 10);
        }
        return result;
    }
}
exports.default = Mapper;
